@@ -89,7 +89,7 @@ List REGISTER2(S4 a, S4 b) {
   
   NumericVector tmp, tmp_p;
   int doit=0;
-  if ((abs(p1.size()-p2.size())>0)){
+  if (p1.size()!=p2.size()){
     doit=1;
   }else{
     if (sum(abs(p1-p2))>1e-20) {doit=1;}
@@ -1090,7 +1090,7 @@ List c_STEP_3_AFFECT_ADA_KMEANS(NumericMatrix lambdas,
   NumericMatrix DiToClu(ind,k);
   int indiv,cluster,variable;
   NumericVector IDX(ind);
-  double SSQ;
+  double SSQ=0;
   
   for (indiv=0;indiv<ind;indiv++){
     for (cluster=0;cluster<k;cluster++){
@@ -1103,8 +1103,8 @@ List c_STEP_3_AFFECT_ADA_KMEANS(NumericMatrix lambdas,
     IDX[indiv]=which_min(DiToClu(indiv,_))+1;
     SSQ=SSQ+(min(DiToClu(indiv,_)));    
   }
-  List resu = List::create(Named("DiToClu") = DiToClu , _["IDX"] = IDX,_["SSQ"]=SSQ);
-  return resu;
+   List resu = List::create(Named("DiToClu") = DiToClu , _["IDX"] = IDX,_["SSQ"]=SSQ);
+   return resu;
 }
 /////////////////
 // [[Rcpp::export]]
